package app;

import java.lang.reflect.InvocationTargetException;

import javax.swing.SwingUtilities;

import dht.*;
import network.*;
import view.*;
import model.*;
import file.*;

/**
 * The driver class that contains the main method and pulls all components together.
 * @author Alex
 * @version 0.2
 * @since 0.1
 */
public class ClientApp {
    
    public static void main(String[] args) {
       
        ClientNetwork network = new ClientNetwork();
        ClientModel model = new ClientModel();
        ClientConsole console = new ClientConsole();
        UDPPacketManager udp = new UDPPacketManager();
        FileManager file = new FileManager();
        ClientLogin login = new ClientLogin();
        ClientView view = new ClientView();
        DHTNetwork dht = new DHTNetwork();
        
        model.initNetwork(network);
        model.initConsole(console);
        model.initView(view);
        model.initDHT(dht);
        console.initModel(model);
        network.initModel(model);
        network.initConsole(console);
        network.initUDP(udp);
        network.initSocket();
        network.initFile(file);
        network.initDHT(dht);
        login.initNetwork(network);
        login.initView(view);
        login.initModel(model);
        login.initDHT(dht);
        view.initModel(model);
        view.refreshPeerList(model.getPeerList());
        view.initDHT(dht);
        view.initUDP(udp);
        dht.initNetwork(network);
        file.initConsole(console);
     
        if(args.length == 4)
		{
        	network.setPresDetails(args[2], args[3]);
			network.setPresState(true);
			model.setServer(args[0], args[1]);
		}
        
        new Thread(network).start();
    }

}
